@@ -235,7 +235,7 @@ static void do_vnc_framebuffer_update(VncConnection *conn,
                         capture->output);
         }
         vnc_connection_shutdown(conn);
-        g_main_quit(capture->loop);
+        g_main_loop_quit(capture->loop);
     }
 }
 
@@ -339,7 +339,7 @@ static void do_vnc_disconnected(VncConnection *conn G_GNUC_UNUSED,
         else
             g_print("Unable to connect to %s:%d\n", capture->host, capture->port - 5900);
     }
-    g_main_quit(capture->loop);
+    g_main_loop_quit(capture->loop);
 }
 
 static void do_vnc_auth_credential(VncConnection *conn, GValueArray *credList, gpointer opaque)
@@ -554,11 +554,3 @@ int main(int argc, char **argv)
 
     return ret;
 }
-
-/*
- * Local variables:
- *  c-indent-level: 4
- *  c-basic-offset: 4
- *  indent-tabs-mode: nil
- * End:
- */
